import os

from django.conf import settings
from rest_framework import serializers
from rest_framework.validators import UniqueValidator
from .models import Imports
from django.contrib.auth import get_user_model

class ImportsSerializer(serializers.ModelSerializer):
    class Meta:
        model = Imports
        #fileds = ('id', 'project_name', 'creation_date')
        fields = '__all__'

class ExportSerializer(serializers.ModelSerializer):
    class Meta:
        model = Imports
        #fileds = ('id', 'project_name', 'creation_date')
        fields = '__all__'


class CreateConversionSerializer(serializers.ModelSerializer):
    file = serializers.FileField()
    project_name = serializers.CharField(min_length=2)
    
    class Meta:
        model = Imports
        fields = ['project_name', 'user_id', 'creation_date', 'file']

    def validate_file(self, value):
        if value.size > 1048576 or value.size == 0:
            raise serializers.ValidationError("File size should not be greater than 1MB")
        return value

    def create(self, validated_data):
        file = validated_data.pop('file')
        import_file = Imports.objects.create(**validated_data)

        # Create directory
        project_directory = os.path.join(settings.MEDIA_ROOT, f'conversions/{validated_data["user_id"]}/{validated_data["project_name"]}')
        if not os.path.exists(project_directory):
            os.makedirs(project_directory)
        # Save file
        with open(os.path.join(project_directory, file.name), 'wb+') as destination:
            for chunk in file.chunks():
                destination.write(chunk)
        # Assign file to import_file
        import_file.file.name = os.path.join(f'conversions/{validated_data["user_id"]}/{validated_data["project_name"]}', file.name)
        import_file.save()

        return import_file

# accounts 
User = get_user_model()

class UserSerializer(serializers.ModelSerializer):
    username = serializers.CharField(max_length=32, validators=[UniqueValidator(queryset=User.objects.all())])
    password = serializers.CharField(min_length=8, write_only=True)

    def create(self, validated_data):
        user = User.objects.create_user(validated_data['username'], validated_data['email'],
             validated_data['password'])
        return user

    class Meta:
        model = User
        fields = ['id', 'username', 'email', 'password']