import json
import logging
import shutil
from datetime import datetime, timedelta
from rest_framework import generics, status, views
from .serializers import ImportsSerializer, CreateConversionSerializer, UserSerializer
from .models import Imports
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.core import serializers
from rest_framework import viewsets
from rest_framework.authentication import TokenAuthentication
from rest_framework.parsers import MultiPartParser, FormParser
from cat_api.getCat import getCat
from handleStorage.convertToJSON import convertXML

logger = logging.getLogger("views")

class ImportsView(viewsets.ModelViewSet):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ImportsSerializer
    def get_queryset(self):
        user = self.request.user
        return Imports.objects.filter(user_id=user)

class RenameImports(views.APIView):
    def put(self, request):
        import_obj = Imports.objects.get(id=request.data.get('id'))
        import_obj.project_name = request.data.get('project_name')
        import_obj.save()
        return Response({"message": "Project name has been updated"})

class ActivateImportsView(views.APIView):
    def put(self, request):
        import_obj = Imports.objects.get(id=request.data.get('id'))
        import_obj.project_name = request.data.get('project_name')
        import_obj.save()
        return Response({"message": "Project name has been updated"})

class ExportXML(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    def get(self, request):
        user = self.request.user
        imports = Imports.objects.filter(user_id=user)
        xml_data = serializers.serialize('xml', imports)
        response = HttpResponse(xml_data, content_type='application/xml')
        response['Content-Disposition'] = 'attachment; filename="imports.xml"'
        return response

class ExportJSON(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    #serializer_class = ExportSerializer
    def get(self, request, import_id):
        import_instance = Imports.objects.get(id=import_id)
        print(import_instance.creation_date)

        json_data = convertXML(import_instance.file.name, import_instance.creation_date, import_instance.paid)

        if json_data == 402:
            return Response(status=status.HTTP_402_PAYMENT_REQUIRED)
        elif json_data == 400:
            return Response(status=status.HTTP_400_BAD_REQUEST)
        else:
            response = HttpResponse(json.dumps(json_data), content_type='application/json')
            response['Content-Disposition'] = 'attachment; filename="imports.json"'
            return response

class CreateImportsView(generics.CreateAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    parser_classes = (MultiPartParser, FormParser)
    serializer_class = CreateConversionSerializer

    def post(self, request, *args, **kwargs):
        getCat()
        user = User.objects.get(id=self.request.data.get('user_id'))
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save(user_id=user, project_name=self.request.data.get('project_name'))
        return Response(serializer.data, status=status.HTTP_201_CREATED)

class DeleteImports(views.APIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    def delete(self, request, import_id):
        try:
            import_instance = Imports.objects.get(id=import_id)
        except Imports.DoesNotExist as err:
            logger.error("Logger: Import not found.")
            return Response({"error": "Import not found."}, status=status.HTTP_404_NOT_FOUND)
        tmp = import_instance.file.name
        import_instance.file.delete()
        import_instance.delete()
        shutil.rmtree("media/" + tmp.rsplit('/', 1)[0] + "/")
        
        
        return Response(status=status.HTTP_204_NO_CONTENT)


class MeView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request):
        serializer = UserSerializer(request.user)
        return Response(serializer.data)

class UserCreate(APIView):
    """ 
    Creates the user. 
    """

    def post(self, request, format='json'):
            serializer = UserSerializer(data=request.data)
            if serializer.is_valid():
                user = serializer.save()
                if user:
                    print("All users:")
                    for userInDB in User.objects.raw('SELECT id, username FROM auth_user'):
                        print(userInDB)
                    return Response(serializer.data, status=status.HTTP_201_CREATED)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
