from django.test import TestCase
from datetime import datetime, timedelta
from .convertToJSON import convertXML
import pytz
import json
import os

class ConvertXMLTest(TestCase):
    def test_convertXML(self):
        # Test case where creation date is more than a day old and paid is 0
        creation_date = datetime.now(pytz.utc) - timedelta(days=2)
        paid = 0
        file_name = "test.xml"
        with open("./media/" + file_name, "w") as xml_file:
            xml_file.write("<root><element>test</element></root>")
            xml_file.close()
        response = convertXML(file_name, creation_date, paid)
        self.assertEqual(response, 402)
        os.remove("./media/" + file_name)

        # Test case where the xml file is not valid
        creation_date = datetime.now(pytz.utc)
        paid = 1
        file_name = "test.xml"
        with open("./media/" + file_name, "w") as xml_file:
            xml_file.write("invalid xml")
            xml_file.close()
        response = convertXML(file_name, creation_date, paid)
        self.assertEqual(response, 400)
        os.remove("./media/" + file_name)

        # Test case where the xml file is valid
        creation_date = datetime.now(pytz.utc)
        paid = 1
        file_name = "test.xml"
        with open("./media/" + file_name, "w") as xml_file:
            xml_file.write("<root><element>test</element></root>")
            xml_file.close()
        response = convertXML(file_name, creation_date, paid)
        expected_response = json.dumps({'root': {'element': 'test'}})
        self.assertEqual(response, expected_response)
        os.remove("./media/" + file_name)
